@@ -52,6 +52,9 @@ from healpy.fitsfunc import write_map
 import nenupy
 from nenupy.instru import nenufar_loc
 
+import logging
+log = logging.getLogger(__name__)
+
 
 # ============================================================= #
 # -------------------------- HpxSky --------------------------- #
@@ -119,6 +122,14 @@ class HpxSky(object):
         
         self._get_nside()
 
+        self._resolution = Angle(
+            angle=nside2resol(
+                self.nside,
+                arcmin=True
+            ),
+            unit=u.arcmin
+        )
+
         self._get_eq_coords()
         return
 
@@ -283,6 +294,7 @@ class HpxSky(object):
         ra_0 = phase_center.ra.rad
         dec_0 = phase_center.dec.rad
         ra_delta = ra - ra_0
+        # ra_delta = ra_0 - ra
         l = np.cos(dec)*np.sin(ra_delta)
         m = np.sin(dec)*np.cos(dec_0) -\
             np.cos(dec)*np.sin(dec_0)*np.cos(ra_delta)
@@ -426,6 +438,9 @@ class HpxSky(object):
             :param grid:
                 Show the equatorial grid.
             :type grid: `bool`
+            :param cbar:
+                Plot a colorbar.
+            :type cbar: `bool`
             :param center:
                 Center of the plot. Default is
                 ``SkyCoord(0.*u.deg, 0.*u.deg)``.
@@ -446,11 +461,18 @@ class HpxSky(object):
                 made on the desired equatorial coordinates:
                 ``(ra (deg), dec (deg), size, color)``.
             :type scatter: `tuple`
+            :param text:
+                Default is ``None``. If not, text is overplotted
+                on the desired equatorial coordinates:
+                ``(ra (deg), dec (deg), text, color)``.
+            :type text: `tuple`
             :param curve:
                 Default is ``None``. If not, a curve plot is
                 made on the desired equatorial coordinates:
                 ``(ra (deg), dec (deg), linestyle, color)``.
             :type curve: `tuple`
+            :param contour: ...
+            :type contour: ...
 
         """
         # Lot of imports for this one...
@@ -536,6 +558,8 @@ class HpxSky(object):
             kwargs['cblabel'] = cblabel
         if 'grid' not in kwargs.keys():
             kwargs['grid'] = True
+        if 'cbar' not in kwargs.keys():
+            kwargs['cbar'] = True
         if 'figsize' not in kwargs.keys():
             kwargs['figsize'] = (15, 10)
         if 'indices' not in kwargs.keys():
@@ -544,6 +568,11 @@ class HpxSky(object):
             kwargs['scatter'] = None
         if 'curve' not in kwargs.keys():
             kwargs['curve'] = None
+        if 'contour' not in kwargs.keys():
+            kwargs['contour'] = None
+        if 'text' not in kwargs.keys():
+            kwargs['text'] = None
+
 
         # Initialize figure
         fig = plt.figure(figsize=kwargs['figsize'])
@@ -608,33 +637,55 @@ class HpxSky(object):
                 color=kwargs['curve'][3],
                 transform=ax.get_transform('world')
             )
+        if kwargs['text'] is not None:
+            for i in range(len(kwargs['text'][0])):
+                ax.text(
+                    x=kwargs['text'][0][i],
+                    y=kwargs['text'][1][i],
+                    s=kwargs['text'][2][i],
+                    color=kwargs['text'][3],
+                    transform=ax.get_transform('world')
+                )
+        if kwargs['contour'] is not None:
+            contour, fp = reproject_from_healpix(
+                (kwargs['contour'][0], ICRS()),
+                wcs,
+                nested=False,
+                shape_out=(ndec, nra)
+            )
+            ax.contour(
+                contour,
+                levels=kwargs['contour'][1],
+                cmap=kwargs['contour'][2],
+            )
 
         im.set_clip_path(ax.coords.frame.patch)
         ax.set_title(kwargs['title'], pad=25)
 
         # Colorbar
-        cax = inset_axes(
-            ax,
-            width='3%',
-            height='100%',
-            loc='lower left',
-            bbox_to_anchor=(1.05, 0., 1, 1),
-            bbox_transform=ax.transAxes,
-            borderpad=0,
-        )
-        cb = ColorbarBase(
-            cax,
-            cmap=get_cmap(name=kwargs['cmap']),
-            orientation='vertical',
-            norm=Normalize(
-                vmin=kwargs['vmin'],
-                vmax=kwargs['vmax']
-            ),
-            ticks=LinearLocator()
-        )
-        cb.solids.set_edgecolor('face')
-        cb.set_label(kwargs['cblabel'])
-        cb.formatter.set_powerlimits((0, 0))
+        if kwargs['cbar']:
+            cax = inset_axes(
+                ax,
+                width='3%',
+                height='100%',
+                loc='lower left',
+                bbox_to_anchor=(1.05, 0., 1, 1),
+                bbox_transform=ax.transAxes,
+                borderpad=0,
+            )
+            cb = ColorbarBase(
+                cax,
+                cmap=get_cmap(name=kwargs['cmap']),
+                orientation='vertical',
+                norm=Normalize(
+                    vmin=kwargs['vmin'],
+                    vmax=kwargs['vmax']
+                ),
+                ticks=LinearLocator()
+            )
+            cb.solids.set_edgecolor('face')
+            cb.set_label(kwargs['cblabel'])
+            cb.formatter.set_powerlimits((0, 0))
 
         # Save or show
         if figname is None:
@@ -652,7 +703,7 @@ class HpxSky(object):
         return
 
 
-    def save(self, filename, header=None):
+    def save(self, filename, header=None, partial=False):
         """ Save the :attr:`~nenupy.astro.hpxsky.HpxSky.skymap`
             to a FITS file, using
             :func:`~healpy.fitsfunc.write_map`. The masked values
@@ -702,7 +753,15 @@ class HpxSky(object):
             coord='C',
             overwrite=True,
             dtype=self.skymap.dtype,
-            extra_header=hd
+            extra_header=hd,
+            partial=partial
+        )
+        log.info(
+            'HEALPix image of {} cells (nside={}) saved in `{}`.'.format(
+                map2write.size,
+                self.nside,
+                filename
+            )
         )
         return
 
