@@ -7,7 +7,7 @@
     BST_Data
     ********
 
-    Object class :class:`.BST_Data` (from the :mod:`ñenupy.beamlet` module) 
+    Object class :class:`.BST_Data` (from the :mod:`~nenupy.beamlet` module) 
     inherits from :class:`.Beamlet`.
     It is designed to read the so called *Beamformed Statistics Data* 
     or `BST <https://nenufar.obs-nancay.fr/en/astronomer/>`_ from the
@@ -646,7 +646,7 @@ class BST_Data(Beamlet):
             )
         ]
         return SData(
-            data=data,
+            data=np.swapaxes(data, 1, 2),
             time=self.time,
             freq=self.freq,
             polar=self.polar
