@@ -21,6 +21,9 @@
     * :func:`~nenupy.instru.instru.sensitivity`: NenuFAR sensitivity
     * :func:`~nenupy.instru.instru.resolution`: NenuFAR resolution
     * :func:`~nenupy.instru.instru.confusion_noise`: NenuFAR confusion noise
+    * :func:`~nenupy.instru.instru.data_rate`: NenuFAR data rate estimation
+    * :func:`~nenupy.instru.instru.freq2sb`: Conversion from frequency to sub-band index
+    * :func:`~nenupy.instru.instru.sb2freq`: Conversion sub-band index to sub-band start frequency
 
 """
 
@@ -36,6 +39,7 @@ __all__ = [
     'ma_info',
     'ma_pos',
     '_HiddenPrints',
+    'getMAL93',
     'nenufar_loc',
     'analog_pointing',
     'desquint_elevation',
@@ -47,7 +51,10 @@ __all__ = [
     'sefd',
     'sensitivity',
     'resolution',
-    'confusion_noise'
+    'confusion_noise',
+    'data_rate',
+    'freq2sb',
+    'sb2freq'
 ]
 
 
@@ -61,13 +68,6 @@ from astropy.coordinates import (
 )
 import astropy.units as u
 from astropy import constants as const
-from healpy import (
-    read_map,
-    ud_grade,
-    nside2npix,
-    pix2ang,
-    Rotator
-)
 import numpy as np
 import os, sys
 from os.path import join, dirname
@@ -231,14 +231,22 @@ class _HiddenPrints:
 # ============================================================= #
 nenufar_loc = EarthLocation(
     lat=47.376511 * u.deg,
-    lon=2.192400 * u.deg,
-    height=150 * u.m
+    lon=2.1924002 * u.deg,
+    height=135.834 * u.m
 )
-# nenufar_loc = EarthLocation( # old
-#     lat=47.375944 * u.deg,
-#     lon=2.193361 * u.deg,
-#     height=136.195 * u.m
-# )
+# ============================================================= #
+
+
+# ============================================================= #
+# -------------------------- getMAL93 ------------------------- #
+# ============================================================= #
+def getMAL93(m):
+    """
+    """
+    ma_pos = np.array([a.tolist() for a in ma_info['pos']])
+    available_mas = np.arange(ma_pos.shape[0])
+    antpos = ma_pos[np.isin(available_mas, m)]
+    return antpos
 # ============================================================= #
 
 
@@ -365,7 +373,7 @@ def desquint_elevation(elevation, opt_freq=30):
 # ============================================================= #
 # --------------------- nenufar_ant_gain ---------------------- #
 # ============================================================= #
-def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
+def nenufar_ant_gain(freq, polar='NW', nside=64, time=None, normalize=True):
     """ Get NenuFAR elementary antenna gain with respect to the
         frequency ``freq``, the polarization ``polar`` and
         convert it to HEALPix representation with a given
@@ -378,7 +386,8 @@ def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
             Frequency of the returned antenna gain. Its value
             must be comprised between 10 and 80 MHz, because
             available antenna models are constrained to these
-            frequencies.
+            frequencies. If the frequency value exceeds 80 MHz,
+            an extrapolation is made using polynomial fits.
         :type freq: `float` or :class:`~astropy.units.Quantity`
         :param polar:
             Antenna polarization to take into account (either
@@ -395,6 +404,9 @@ def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
             NenuFAR. If ``None`` the map is not rotated to 
             equatorial coordinates.
         :type time: `str` or :class:`~astropy.time.Time`
+        :param normalize:
+            Returns the normalized gain or not.
+        :type normalize: `bool`
 
         :returns:
             Sky map in HEALPix representation of normalized
@@ -422,6 +434,14 @@ def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
         .. seealso::
             :class:`~nenupy.beam.hpxbeam.HpxABeam`
     """
+    from healpy import (
+        read_map,
+        ud_grade,
+        nside2npix,
+        pix2ang,
+        Rotator
+    )
+
     # Parameter checks
     if not isinstance(freq, Quantity):
         freq *= u.MHz
@@ -430,10 +450,6 @@ def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
         raise ValueError(
             'Polar should be either NW or NE'
         )
-    if (freq < 10) or (freq > 80):
-        raise ValueError(
-            'frequency should be set between 10 and 80 MHz.'
-        )
     polar = polar.upper()
     # Correspondance between polar/freq and field index in FITS
     gain_freqs = np.arange(10, 90, 10, dtype=int)
@@ -443,35 +459,69 @@ def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
         for f in gain_freqs:
             cols['{}_{}'.format(p, f)] = count
             count += 1
-    # Get Low and High ant gain bounding freq
-    f_low = gain_freqs[gain_freqs <= freq].max()
-    f_high = gain_freqs[gain_freqs >= freq].min()
     antgain_file = join(
         dirname(__file__),
         'NenuFAR_Ant_Hpx.fits',
     )
-    gain_low = read_map(
-        filename=antgain_file,
-        hdu=1,
-        field=cols['{}_{}'.format(polar, f_low)],
-        verbose=False,
-        memmap=True,
-        dtype=float
-    )
-    gain_high = read_map(
-        filename=antgain_file,
-        hdu=1,
-        field=cols['{}_{}'.format(polar, f_high)],
-        verbose=False,
-        memmap=True,
-        dtype=float
-    )
-    # Make interpolation
-    if f_low != f_high:
-        gain = gain_low * (f_high - freq)/10. +\
-            gain_high * (freq - f_low)/10.
+    if freq < 10:
+        raise ValueError(
+            'No antenna model < 10 MHz.'
+        )
+    elif freq > 80:
+        log.warning(
+            'NenuFAR antenna response is extrapolated > 80 MHz.'
+        )
+        # Will fit a polynomial along the high end of frequencies
+        freq_to_fit = np.arange(40, 90, 10, dtype=int)
+        gains = np.zeros((freq_to_fit.size, nside2npix(64)))
+        # Construct the gain map (freqs, npix)
+        for i, f in enumerate(freq_to_fit):
+            gains[i, :] = read_map(
+                filename=antgain_file,
+                hdu=1,
+                field=cols['{}_{}'.format(polar, f)],
+                verbose=False,
+                memmap=True,
+                dtype=float
+            )
+        # Get the polynomial coefficients
+        coeffs = np.polyfit(freq_to_fit, gains, 3)
+        def poly(x, coeffs):
+            """ Retrieve the polynomial from coefficients
+            """
+            na = np.newaxis
+            order = coeffs.shape[0]
+            poly = np.zeros((x.size, coeffs.shape[1]))
+            for deg in range(order):
+                poly += (x**deg)[:, na] * coeffs[order-deg-1, :][na, :]
+            return poly
+        gain = poly(np.array([freq]), coeffs).ravel()
     else:
-        gain = gain_low
+        # Get Low and High ant gain bounding freq
+        f_low = gain_freqs[gain_freqs <= freq].max()
+        f_high = gain_freqs[gain_freqs >= freq].min()
+        gain_low = read_map(
+            filename=antgain_file,
+            hdu=1,
+            field=cols['{}_{}'.format(polar, f_low)],
+            verbose=False,
+            memmap=True,
+            dtype=float
+        )
+        gain_high = read_map(
+            filename=antgain_file,
+            hdu=1,
+            field=cols['{}_{}'.format(polar, f_high)],
+            verbose=False,
+            memmap=True,
+            dtype=float
+        )
+        # Make interpolation
+        if f_low != f_high:
+            gain = gain_low * (f_high - freq)/10. +\
+                gain_high * (freq - f_low)/10.
+        else:
+            gain = gain_low
     # Rotate the map to equatorial coordinates
     if time is not None:
         if not isinstance(time, Time):
@@ -493,7 +543,7 @@ def nenufar_ant_gain(freq, polar='NW', nside=64, time=None):
             gain = rot.rotate_map_alms(gain)
     # Convert HEALPix map to required nside
     gain = ud_grade(gain, nside_out=nside)
-    return gain / gain.max()
+    return gain / gain.max() if normalize else gain
 # ============================================================= #
 
 
@@ -1004,3 +1054,233 @@ def confusion_noise(freq=50, miniarrays=None):
     return conf * u.mJy # mJy/beam
 # ============================================================= #
 
+
+# ============================================================= #
+# ------------------------- data_rate ------------------------- #
+# ============================================================= #
+def data_rate(mode='imaging', mas=96, dt=1, nchan=64, bandwidth=75, nb=1):
+    r""" Estimates the NenuFAR data rate product. To get the total
+        observation size, a simple multiplication with a
+        :class:`~astropy.units.Quantity` time instance,
+        corresponding to total exposure time, is needed. To convert
+        bytes in binary base (i.e. 1 kB = 1024 B), use `astropy`
+        unit conversions (with prefixes ``Ki``, ``Mi``, ``Gi`` or
+        ``Ti`` for KiloBytes MegaBytes, GigaBytes and TeraBytes,
+        see `astropy unit prefixes <https://docs.astropy.org/en/stable/units/standard_units.html#prefixes>`_).
+
+        Data rates (in bytes/s) are computed as follows:
+
+        * Imaging mode (*NICKEL* correlator): :math:`r_{\rm im} = n_{\rm correlations} d_{\rm complex\, 64\, bits} n_{\rm channels} n_{\rm baselines} n_{\rm subbands} / \delta t`;
+        * Beamforming mode (*UnDySPuTeD* backend): :math:`r_{\rm bf} = n_{\rm correlations} d_{\rm float\, 32\, bits} n_{\rm channels} n_{\rm subbands} / \delta t`;
+        * Waveform mode (*LaNewBa* backend): :math:`r_{\rm wf} = 195312.5 \times n_{\rm raw} n_{\rm b} n_{\rm subbands}`;
+        * Transient Buffer mode (*LaNewBa* backend): :math:`r_{\rm tbb} = 195312.5 \times 1024 \times n_{\rm raw} n_{\rm b} n_{\rm mas}`;
+        
+        where :math:`n_{\rm correlations} = 4` (XX, XY, YX, YY),
+        :math:`n_{\rm baselines} = n_{\rm mas}*(n_{\rm mas}-1)/2 + n_{\rm mas}`,
+        :math:`n_{\rm subbands} = \Delta \nu / 195.3125\, \rm{kHz}`,
+        :math:`n_{\rm raw} = 4` (Re(X), Im(X), Re(Y), Im(Y)),
+        :math:`d_{\rm complex\, 64\, bits}` and :math:`d_{\rm float\, 32\, bits}`
+        are data sizes in bytes.
+        
+        :param mode:
+            Observation mode (either ``'imaging'`` or
+            ``'beamforming'`` or ``'waveform'`` or ``'tbb'``).
+        :type mode: `str`
+        :param mas: Number of Mini-Arrays to take into account
+            :math:`n_{\rm mas}`. Default is ``96``.
+        :type mas: `int`
+        :param dt: Observation time step :math:`\delta t` (in
+            seconds if no unit is provided). Default is ``1 sec``.
+        :type dt: `float` or :class:`~astropy.units.Quantity`
+        :param nchan: Number of channels per subband :math:`n_{\rm channels}`.
+            Each subband is 195.3125 kHz. Default is ``64``.
+        :type nchan: `int`
+        :param bandwidth: Observation bandwidth :math:`\Delta \nu`
+            (in MHz if no unit is provided). Default is ``75 MHz``.
+        :type bandwidth: `float` or :class:`~astropy.units.Quantity`
+        :param nb: Number of bytes of raw data samples :math:`n_{\rm b}`
+            (``1 = 8 bits``, ``2 = 16 bits``).
+        :type nb: `int`
+
+        :returns: Data rate in bytes/s.
+        :rtype: :class:`~astropy.units.Quantity`
+
+        :example:
+            Imaging data rate and total size for a 1h exposure, converted in TB:
+
+            >>> from nenupy.instru import data_rate
+            >>> import astropy.units as u
+            >>> rate = data_rate(
+                    mode='imaging',
+                    mas=96,
+                    dt=1*u.s,
+                    nchan=64,
+                    bandwidth=75*u.MHz
+                )
+            >>> print(rate)
+            3.6616274×10^9 byte/s
+            >>> exposure = 3600*u.s
+            >>> size = rate * exposure
+            >>> print(size)
+            1.3181859×10^13 byte
+            >>> print(size.to(u.Tibyte))
+            11.988831 Tibyte
+
+        .. note::
+            IDL original version v1, PZ, 2019-03-19
+            
+            PYTHON transcript v1, JG, 2020-05-09
+            
+            Pythonized for `nenupy`, AL, 2020-05-11
+
+        .. versionadded:: 1.1.0
+    """
+    # Input checks
+    available_modes = ['imaging', 'beamforming', 'waveform', 'tbb']
+    if not mode in available_modes:
+        raise ValueError(
+            'mode should be one of {}'.format(available_modes)
+        )
+    if not isinstance(mas, int):
+        raise TypeError(
+            'mas should be an integer'
+        )
+    if not isinstance(dt, u.Quantity):
+        dt *= u.s
+    if not isinstance(nchan, int):
+        raise TypeError(
+            'nchan should be an integer'
+        )
+    if nchan > 64:
+        raise ValueError(
+            'NenuFAR maximal number of channels per subband is 64'
+        )
+    if not isinstance(bandwidth, u.Quantity):
+        bandwidth *= u.MHz
+    if bandwidth > 150*u.MHz:
+        raise ValueError(
+            'NenuFAR maximal bandwidth is 150 MHz.'
+        )
+
+    # NenuFAR backend properties
+    sb_width = 195.3125*u.kHz # subband bandwidth
+    n_sb_max = 768 # maximal number of subbands
+    n_corr = 4 # XX, XY, YX, YY 
+
+    # Number of sub-bands involved
+    n_sb = int(np.round(bandwidth.to(u.kHz)/sb_width))
+
+    if mode == 'imaging':
+        # Complex in bytes
+        nenucomplex = np.complex64().itemsize * u.byte
+        n_baselines = mas*(mas - 1)/2 + mas
+        rate_sb = n_corr*nenucomplex*nchan*n_baselines/dt
+        rate = rate_sb * n_sb
+    elif mode == 'beamforming':
+        # Floats as bytes in NenuFAR calculators
+        nenufloat = np.float32().itemsize * u.byte
+        rate_sb = n_corr*nenufloat*nchan/dt
+        rate = rate_sb * n_sb
+    elif mode == 'waveform':
+        rate_sb = 4*nb*sb_width.to(u.Hz).value
+        rate = rate_sb * n_sb * u.byte / u.s
+    elif mode == 'tbb':
+        rate = 4*nb*sb_width.to(u.Hz).value*1024*mas  * u.byte / u.s
+
+    return rate
+# ============================================================= #
+
+
+# ============================================================= #
+# ------------------------- freq2sb --------------------------- #
+# ============================================================= #
+def freq2sb(freq):
+    r""" Conversion between the frequency :math:`\nu` and the
+        NenuFAR sub-band index :math:`n_{\rm SB}`.
+        Each NenuFAR sub-band has a bandwidth of
+        :math:`\Delta \nu = 195.3125\, \rm{kHz}`:
+
+        .. math::
+            n_{\rm SB} = \frac{512 \times \nu}{\Delta \nu}
+
+        :param freq:
+            Frequency to convert in sub-band index (assumed in
+            MHz if no unit is provided).
+        :type freq:
+            `float`, :class:`~numpy.ndarray` or :class:`~astropy.units.Quantity`
+
+        :returns:
+            Sub-band index, same dimension as ``freq``.
+        :rtype: `int` or :class:`~numpy.ndarray`
+
+        :example:
+            >>> from nenupy.instru import freq2sb
+            >>> freq2sb(freq=50.5)
+            258
+            >>> freq2sb(freq=[50.5, 51])
+            array([258, 261])
+
+        .. versionadded:: 1.1.0
+    """
+    if not isinstance(freq, u.Quantity):
+        freq *= u.MHz
+    if (freq.min() < 0 * u.MHz) or (freq.max() > 100 * u.MHz):
+        raise ValueError(
+            'freq should be between 0 and 100 MHz.'
+        )
+    freq = freq.to(u.MHz)
+    sb_width = 100. * u.MHz
+    sb_idx = np.floor((freq * 512) / sb_width)
+    return sb_idx.astype(int).value
+# ============================================================= #
+
+
+# ============================================================= #
+# ------------------------- freq2sb --------------------------- #
+# ============================================================= #
+def sb2freq(sb):
+    r""" Conversion between NenuFAR sub-band index :math:`n_{\rm SB}`
+        to sub-band starting frequency :math:`\nu_{\rm start}`:
+
+        .. math::
+            \nu_{\rm start} = \frac{n_{\rm SB} \times \Delta \nu}{512}
+
+        Each NenuFAR sub-band has a bandwidth of
+        :math:`\Delta \nu = 195.3125\, \rm{kHz}`, therefore, the
+        sub-band :math:`n_{\rm SB}` goes from :math:`\nu_{\rm start}`
+        to :math:`\nu_{\rm stop} = \nu_{\rm start} + \Delta \nu`.
+
+        :param sb:
+            Sub-band index (from 0 to 511).
+        :type sb: `int` or :class:`~numpy.ndarray` of `int`
+
+        :returns:
+            Sub-band start frequency :math:`\nu_{\rm start}` in MHz.
+        :rtype: :class:`~astropy.units.Quantity`
+
+        :example:
+            >>> from nenupy.instru import sb2freq
+            >>> sb2freq(sb=1)
+            [0.1953125] MHz
+            >>> sb2freq(sb=[1, 2, 3, 4])
+            [0.1953125, 0.390625, 0.5859375, 0.78125] MHz
+
+        .. versionadded:: 1.1.0
+    """
+    if np.isscalar(sb):
+        sb = np.array([sb])
+    else:
+        sb = np.array(sb)
+    if sb.dtype.name not in ['int32', 'int64']:
+        raise TypeError(
+            'sb should be integers.'
+        )
+    if (sb.min() < 0) or (sb.max() > 511):
+        raise ValueError(
+            'sb should be between 0 and 511.'
+        )
+    sb_width = 100. * u.MHz
+    freq_start = (sb * sb_width) / 512
+    return freq_start
+# ============================================================= #
+
